'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = pipeStreams;

var _cliProgress = require('cli-progress');

var _cliProgress2 = _interopRequireDefault(_cliProgress);

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _formatSize = require('./format-size');

var _logger = require('./logger');

var _logger2 = _interopRequireDefault(_logger);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

function pipeStreams(readStream, writeStream, filesize, label) {
  return new Promise((() => {
    var _ref = _asyncToGenerator(function* (resolve) {

      const unit = (0, _formatSize.getUnit)(filesize);
      const unitLabel = (0, _formatSize.getUnitLabel)(unit);

      const labelWidth = (process.stdout.columns / 3).toFixed();
      label = label.substr(0, labelWidth).padEnd(labelWidth, ' ');

      const progress = new _cliProgress2.default.Bar({
        format: `${label} [${_chalk2.default.green('{bar}')}] {percentage}% | {duration_formatted} | {transfer}{unitLabel}`
      }, _cliProgress2.default.Presets.legacy);

      progress.start(filesize, 0, {
        transfer: `${(0, _formatSize.formatSize)(0, { unit })} / ${(0, _formatSize.formatSize)(filesize, { unit })}`,
        unitLabel
      });

      readStream.on('data', function (buffer) {
        progress.increment(buffer.length, {
          transfer: `${(0, _formatSize.formatSize)(progress.value, { unit })}/${(0, _formatSize.formatSize)(filesize, { unit })}`
        });
      });

      readStream.on('error', function (err) {
        progress.stop();
        _logger2.default.error(err.toString());
        _logger2.default.error('Failed to read file.');
        process.exit(1);
      });

      readStream.pipe(writeStream);

      writeStream.on('finish', function () {
        progress.update(filesize, {
          transfer: (0, _formatSize.formatSize)(filesize, { unit })
        });
        progress.stop();
        resolve();
      });

      writeStream.on('error', function (err) {
        progress.stop();
        _logger2.default.error(err.toString());
        _logger2.default.error('Failed to write file to archive.');
        process.exit(1);
      });
    });

    return function (_x) {
      return _ref.apply(this, arguments);
    };
  })());
}